# -*- coding: utf-8 -*-
'''
    :codeauthor: Rajvi Dhimar <rajvidhimar95@gmail.com>
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import test libs
from tests.support.mixins import LoaderModuleMockMixin, XMLEqualityMixin
from tests.support.mock import patch, mock_open
from tests.support.unit import skipIf, TestCase

# Import 3rd-party libs
try:
    from lxml import etree
except ImportError:
    from salt._compat import ElementTree as etree

try:
    from jnpr.junos.utils.config import Config
    from jnpr.junos.utils.sw import SW
    from jnpr.junos.device import Device
    from jnpr.junos.device import Device
    import jxmlease  # pylint: disable=unused-import
    HAS_JUNOS = True
except ImportError:
    HAS_JUNOS = False

# Import salt modules
import salt.modules.junos as junos


@skipIf(not HAS_JUNOS, 'The junos-eznc and jxmlease modules are required')
class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):

    def setup_loader_modules(self):
        return {
            junos: {
                '__proxy__': {
                    'junos.conn': self.make_connect,
                    'junos.get_serialized_facts': self.get_facts
                },
                '__salt__': {'cp.get_template': self.mock_cp,
                             'cp.get_file': self.mock_cp}
            }
        }

    def mock_cp(self, *args, **kwargs):
        pass

    def make_connect(self):
        with patch('ncclient.manager.connect') as mock_connect:
            self.dev = self.dev = Device(
                host='1.1.1.1',
                user='test',
                password='test123',
                fact_style='old',
                gather_facts=False)
            self.dev.open()
            self.dev.timeout = 30
            self.dev.bind(cu=Config)
            self.dev.bind(sw=SW)
            self.addCleanup(delattr, self, 'dev')
            return self.dev

    def raise_exception(self, *args, **kwargs):
        raise Exception('Test exception')

    def get_facts(self):
        facts = {'2RE': True,
                 'HOME': '/var/home/regress',
                 'RE0': {'last_reboot_reason': '0x200:normal shutdown',
                         'mastership_state': 'master',
                         'model': 'RE-VMX',
                         'status': 'OK',
                         'up_time': '11 days, 23 hours, 16 minutes, 54 seconds'},
                 'RE1': {'last_reboot_reason': '0x200:normal shutdown',
                         'mastership_state': 'backup',
                         'model': 'RE-VMX',
                         'status': 'OK',
                         'up_time': '11 days, 23 hours, 16 minutes, 41 seconds'},
                 'RE_hw_mi': False,
                 'current_re': ['re0', 'master', 'node', 'fwdd', 'member', 'pfem'],
                 'domain': 'englab.juniper.net',
                 'fqdn': 'R1_re0.englab.juniper.net',
                 'hostname': 'R1_re0',
                 'hostname_info': {'re0': 'R1_re0', 're1': 'R1_re01'},
                 'ifd_style': 'CLASSIC',
                 'junos_info': {'re0': {'object': {'build': None,
                                                   'major': (16, 1),
                                                   'minor': '20160413_0837_aamish',
                                                   'type': 'I'},
                                        'text': '16.1I20160413_0837_aamish'},
                                're1': {'object': {'build': None,
                                                   'major': (16, 1),
                                                   'minor': '20160413_0837_aamish',
                                                   'type': 'I'},
                                        'text': '16.1I20160413_0837_aamish'}},
                 'master': 'RE0',
                 'model': 'MX240',
                 'model_info': {'re0': 'MX240', 're1': 'MX240'},
                 'personality': 'MX',
                 're_info': {'default': {'0': {'last_reboot_reason': '0x200:normal shutdown',
                                               'mastership_state': 'master',
                                               'model': 'RE-VMX',
                                               'status': 'OK'},
                                         '1': {'last_reboot_reason': '0x200:normal shutdown',
                                               'mastership_state': 'backup',
                                               'model': 'RE-VMX',
                                               'status': 'OK'},
                                         'default': {'last_reboot_reason': '0x200:normal shutdown',
                                                     'mastership_state': 'master',
                                                     'model': 'RE-VMX',
                                                     'status': 'OK'}}},
                 're_master': {'default': '0'},
                 'serialnumber': 'VMX4eaf',
                 'srx_cluster': None,
                 'switch_style': 'BRIDGE_DOMAIN',
                 'vc_capable': False,
                 'vc_fabric': None,
                 'vc_master': None,
                 'vc_mode': None,
                 'version': '16.1I20160413_0837_aamish',
                 'version_RE0': '16.1I20160413_0837_aamish',
                 'version_RE1': '16.1I20160413_0837_aamish',
                 'version_info': {'build': None,
                                  'major': (16, 1),
                                  'minor': '20160413_0837_aamish',
                                  'type': 'I'},
                 'virtual': True}
        return facts

    def test_facts_refresh(self):
        with patch('salt.modules.saltutil.sync_grains') as mock_sync_grains:
            ret = dict()
            ret['facts'] = {'2RE': True,
                            'HOME': '/var/home/regress',
                            'RE0': {'last_reboot_reason': '0x200:normal shutdown',
                                    'mastership_state': 'master',
                                    'model': 'RE-VMX',
                                    'status': 'OK',
                                    'up_time': '11 days, 23 hours, 16 minutes, 54 seconds'},
                            'RE1': {'last_reboot_reason': '0x200:normal shutdown',
                                    'mastership_state': 'backup',
                                    'model': 'RE-VMX',
                                    'status': 'OK',
                                    'up_time': '11 days, 23 hours, 16 minutes, 41 seconds'},
                            'RE_hw_mi': False,
                            'current_re': ['re0', 'master', 'node', 'fwdd', 'member', 'pfem'],
                            'domain': 'englab.juniper.net',
                            'fqdn': 'R1_re0.englab.juniper.net',
                            'hostname': 'R1_re0',
                            'hostname_info': {'re0': 'R1_re0', 're1': 'R1_re01'},
                            'ifd_style': 'CLASSIC',
                            'junos_info': {'re0': {'object': {'build': None,
                                                              'major': (16, 1),
                                                              'minor': '20160413_0837_aamish',
                                                              'type': 'I'},
                                                   'text': '16.1I20160413_0837_aamish'},
                                           're1': {'object': {'build': None,
                                                              'major': (16, 1),
                                                              'minor': '20160413_0837_aamish',
                                                              'type': 'I'},
                                                   'text': '16.1I20160413_0837_aamish'}},
                            'master': 'RE0',
                            'model': 'MX240',
                            'model_info': {'re0': 'MX240', 're1': 'MX240'},
                            'personality': 'MX',
                            're_info': {'default': {'0': {'last_reboot_reason': '0x200:normal shutdown',
                                                          'mastership_state': 'master',
                                                          'model': 'RE-VMX',
                                                          'status': 'OK'},
                                                    '1': {'last_reboot_reason': '0x200:normal shutdown',
                                                          'mastership_state': 'backup',
                                                          'model': 'RE-VMX',
                                                          'status': 'OK'},
                                                    'default': {'last_reboot_reason': '0x200:normal shutdown',
                                                                'mastership_state': 'master',
                                                                'model': 'RE-VMX',
                                                                'status': 'OK'}}},
                            're_master': {'default': '0'},
                            'serialnumber': 'VMX4eaf',
                            'srx_cluster': None,
                            'switch_style': 'BRIDGE_DOMAIN',
                            'vc_capable': False,
                            'vc_fabric': None,
                            'vc_master': None,
                            'vc_mode': None,
                            'version': '16.1I20160413_0837_aamish',
                            'version_RE0': '16.1I20160413_0837_aamish',
                            'version_RE1': '16.1I20160413_0837_aamish',
                            'version_info': {'build': None,
                                             'major': (16, 1),
                                             'minor': '20160413_0837_aamish',
                                             'type': 'I'},
                            'virtual': True}
            ret['out'] = True
            self.assertEqual(junos.facts_refresh(), ret)

    def test_facts_refresh_exception(self):
        with patch('jnpr.junos.device.Device.facts_refresh') as mock_facts_refresh:
            mock_facts_refresh.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Execution failed due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.facts_refresh(), ret)

    def test_facts(self):
        ret = dict()
        ret['facts'] = {'2RE': True,
                        'HOME': '/var/home/regress',
                        'RE0': {'last_reboot_reason': '0x200:normal shutdown',
                                'mastership_state': 'master',
                                'model': 'RE-VMX',
                                'status': 'OK',
                                'up_time': '11 days, 23 hours, 16 minutes, 54 seconds'},
                        'RE1': {'last_reboot_reason': '0x200:normal shutdown',
                                'mastership_state': 'backup',
                                'model': 'RE-VMX',
                                'status': 'OK',
                                'up_time': '11 days, 23 hours, 16 minutes, 41 seconds'},
                        'RE_hw_mi': False,
                        'current_re': ['re0', 'master', 'node', 'fwdd', 'member', 'pfem'],
                        'domain': 'englab.juniper.net',
                        'fqdn': 'R1_re0.englab.juniper.net',
                        'hostname': 'R1_re0',
                        'hostname_info': {'re0': 'R1_re0', 're1': 'R1_re01'},
                        'ifd_style': 'CLASSIC',
                        'junos_info': {'re0': {'object': {'build': None,
                                                          'major': (16, 1),
                                                          'minor': '20160413_0837_aamish',
                                                          'type': 'I'},
                                               'text': '16.1I20160413_0837_aamish'},
                                       're1': {'object': {'build': None,
                                                          'major': (16, 1),
                                                          'minor': '20160413_0837_aamish',
                                                          'type': 'I'},
                                               'text': '16.1I20160413_0837_aamish'}},
                        'master': 'RE0',
                        'model': 'MX240',
                        'model_info': {'re0': 'MX240', 're1': 'MX240'},
                        'personality': 'MX',
                        're_info': {'default': {'0': {'last_reboot_reason': '0x200:normal shutdown',
                                                      'mastership_state': 'master',
                                                      'model': 'RE-VMX',
                                                      'status': 'OK'},
                                                '1': {'last_reboot_reason': '0x200:normal shutdown',
                                                      'mastership_state': 'backup',
                                                      'model': 'RE-VMX',
                                                      'status': 'OK'},
                                                'default': {'last_reboot_reason': '0x200:normal shutdown',
                                                            'mastership_state': 'master',
                                                            'model': 'RE-VMX',
                                                            'status': 'OK'}}},
                        're_master': {'default': '0'},
                        'serialnumber': 'VMX4eaf',
                        'srx_cluster': None,
                        'switch_style': 'BRIDGE_DOMAIN',
                        'vc_capable': False,
                        'vc_fabric': None,
                        'vc_master': None,
                        'vc_mode': None,
                        'version': '16.1I20160413_0837_aamish',
                        'version_RE0': '16.1I20160413_0837_aamish',
                        'version_RE1': '16.1I20160413_0837_aamish',
                        'version_info': {'build': None,
                                         'major': (16, 1),
                                         'minor': '20160413_0837_aamish',
                                         'type': 'I'},
                        'virtual': True}
        ret['out'] = True
        self.assertEqual(junos.facts(), ret)

    def test_facts_exception(self):
        with patch.dict(junos.__proxy__, {'junos.get_serialized_facts': self.raise_exception}):
            ret = dict()
            ret['message'] = 'Could not display facts due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.facts(), ret)

    def test_set_hostname_without_args(self):
        ret = dict()
        ret['message'] = 'Please provide the hostname.'
        ret['out'] = False
        self.assertEqual(junos.set_hostname(), ret)

    def test_set_hostname_load_called_with_valid_name(self):
        with patch('jnpr.junos.utils.config.Config.load') as mock_load:
            junos.set_hostname('test-name')
            mock_load.assert_called_with(
                'set system host-name test-name', format='set')

    def test_set_hostname_raise_exception_for_load(self):
        with patch('jnpr.junos.utils.config.Config.load') as mock_load:
            mock_load.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not load configuration due to error "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.set_hostname('Test-name'), ret)

    def test_set_hostname_raise_exception_for_commit_check(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check:
            mock_commit_check.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not commit check due to error "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.set_hostname('test-name'), ret)

    def test_set_hostname_one_arg_parsed_correctly(self):
        with patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit_check.return_value = True
            args = {'comment': 'Committed via salt', '__pub_user': 'root',
                    '__pub_arg': ['test-name', {'comment': 'Committed via salt'}],
                    '__pub_fun': 'junos.set_hostname', '__pub_jid':
                        '20170220210915624885', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}

            junos.set_hostname('test-name', **args)
            mock_commit.assert_called_with(comment='Committed via salt')

    def test_set_hostname_more_than_one_args_parsed_correctly(self):
        with patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit_check.return_value = True
            args = {'comment': 'Committed via salt',
                    '__pub_user': 'root',
                    '__pub_arg': ['test-name',
                                  {'comment': 'Committed via salt',
                                   'confirm': 5}],
                    '__pub_fun': 'junos.set_hostname',
                    '__pub_jid': '20170220210915624885',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}

            junos.set_hostname('test-name', **args)
            mock_commit.assert_called_with(comment='Committed via salt', confirm=5)

    def test_set_hostname_successful_return_message(self):
        with patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit_check.return_value = True
            args = {'comment': 'Committed via salt',
                    '__pub_user': 'root',
                    '__pub_arg': ['test-name',
                                  {'comment': 'Committed via salt'}],
                    '__pub_fun': 'junos.set_hostname',
                    '__pub_jid': '20170220210915624885',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            ret = dict()
            ret['message'] = 'Successfully changed hostname.'
            ret['out'] = True
            self.assertEqual(junos.set_hostname('test-name', **args), ret)

    def test_set_hostname_raise_exception_for_commit(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Successfully loaded host-name but commit failed with "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.set_hostname('test-name'), ret)

    def test_set_hostname_fail_commit_check(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('salt.modules.junos.rollback') as mock_rollback:
            mock_commit_check.return_value = False
            ret = dict()
            ret['out'] = False
            ret['message'] = 'Successfully loaded host-name but pre-commit check failed.'
            self.assertEqual(junos.set_hostname('test'), ret)

    def test_commit_without_args(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit.return_value = True
            mock_commit_check.return_value = True
            ret = dict()
            ret['message'] = 'Commit Successful.'
            ret['out'] = True
            self.assertEqual(junos.commit(), ret)

    def test_commit_raise_commit_check_exeception(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check:
            mock_commit_check.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not perform commit check due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.commit(), ret)

    def test_commit_raise_commit_exception(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit_check.return_value = True
            mock_commit.side_effect = self.raise_exception
            ret = dict()
            ret['out'] = False
            ret['message'] = \
                'Commit check succeeded but actual commit failed with "Test exception"'
            self.assertEqual(junos.commit(), ret)

    def test_commit_with_single_argument(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit_check.return_value = True
            args = {'__pub_user': 'root',
                    '__pub_arg': [{'sync': True}],
                    'sync': True,
                    '__pub_fun': 'junos.commit',
                    '__pub_jid': '20170221182531323467',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.commit(**args)
            mock_commit.assert_called_with(detail=False, sync=True)

    def test_commit_with_multiple_arguments(
            self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit_check.return_value = True
            args = {'comment': 'comitted via salt',
                    '__pub_user': 'root',
                    '__pub_arg': [{'comment': 'comitted via salt',
                                   'confirm': 3,
                                   'detail': True}],
                    'confirm': 3,
                    'detail': True,
                    '__pub_fun': 'junos.commit',
                    '__pub_jid': '20170221182856987820',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.commit(**args)
            mock_commit.assert_called_with(
                comment='comitted via salt', detail=True, confirm=3)

    def test_commit_pyez_commit_returning_false(
            self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit:
            mock_commit.return_value = False
            mock_commit_check.return_value = True
            ret = dict()
            ret['message'] = 'Commit failed.'
            ret['out'] = False
            self.assertEqual(junos.commit(), ret)

    def test_commit_pyez_commit_check_returns_false(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check:
            mock_commit_check.return_value = False
            ret = dict()
            ret['out'] = False
            ret['message'] = 'Pre-commit check failed.'
            self.assertEqual(junos.commit(), ret)

    def test_rollback_exception(self):
        with patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_rollback.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Rollback failed due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.rollback(), ret)

    def test_rollback_without_args_success(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            mock_rollback.return_value = True
            ret = dict()
            ret['message'] = 'Rollback successful'
            ret['out'] = True
            self.assertEqual(junos.rollback(), ret)

    def test_rollback_without_args_fail(self):
        with patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_rollback.return_value = False
            ret = dict()
            ret['message'] = 'Rollback failed'
            ret['out'] = False
            self.assertEqual(junos.rollback(), ret)

    def test_rollback_with_id(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            junos.rollback(id=5)
            mock_rollback.assert_called_with(5)

    def test_rollback_with_id_and_single_arg(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            args = {'__pub_user': 'root', '__pub_arg': [2, {'confirm': 2}],
                    'confirm': 2, '__pub_fun': 'junos.rollback',
                    '__pub_jid': '20170221184518526067', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}
            junos.rollback(id=2, **args)
            mock_rollback.assert_called_with(2)
            mock_commit.assert_called_with(confirm=2)

    def test_rollback_with_id_and_multiple_args(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            args = {'comment': 'Comitted via salt',
                    '__pub_user': 'root',
                    'dev_timeout': 40,
                    '__pub_arg': [2,
                                  {'comment': 'Comitted via salt',
                                   'timeout': 40,
                                   'confirm': 1}],
                    'confirm': 1,
                    '__pub_fun': 'junos.rollback',
                    '__pub_jid': '20170221192708251721',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.rollback(id=2, **args)
            mock_rollback.assert_called_with(2)
            mock_commit.assert_called_with(
                comment='Comitted via salt', confirm=1, timeout=40)

    def test_rollback_with_only_single_arg(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            args = {'__pub_user': 'root',
                    '__pub_arg': [{'sync': True}],
                    'sync': True,
                    '__pub_fun': 'junos.rollback',
                    '__pub_jid': '20170221193615696475',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.rollback(**args)
            mock_rollback.assert_called_once_with(0)
            mock_commit.assert_called_once_with(sync=True)

    def test_rollback_with_only_multiple_args_no_id(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            args = {'comment': 'Comitted via salt',
                    '__pub_user': 'root',
                    '__pub_arg': [{'comment': 'Comitted via salt',
                                   'confirm': 3,
                                   'sync': True}],
                    'confirm': 3,
                    'sync': True,
                    '__pub_fun': 'junos.rollback',
                    '__pub_jid': '20170221193945996362',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.rollback(**args)
            mock_rollback.assert_called_with(0)
            mock_commit.assert_called_once_with(
                sync=True, confirm=3, comment='Comitted via salt')

    def test_rollback_with_diffs_file_option_when_diff_is_None(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback, \
                patch('salt.utils.files.fopen') as mock_fopen, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
            mock_commit_check.return_value = True
            mock_diff.return_value = 'diff'
            args = {'__pub_user': 'root',
                    '__pub_arg': [{'diffs_file': '/home/regress/diff',
                                   'confirm': 2}],
                    'confirm': 2,
                    '__pub_fun': 'junos.rollback',
                    '__pub_jid': '20170221205153884009',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': '',
                    'diffs_file': '/home/regress/diff'}
            junos.rollback(**args)
            mock_fopen.assert_called_with('/home/regress/diff', 'w')

    def test_rollback_with_diffs_file_option(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback, \
                patch('salt.utils.files.fopen') as mock_fopen, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
            mock_commit_check.return_value = True
            mock_diff.return_value = None
            args = {'__pub_user': 'root',
                    '__pub_arg': [{'diffs_file': '/home/regress/diff',
                                   'confirm': 2}],
                    'confirm': 2,
                    '__pub_fun': 'junos.rollback',
                    '__pub_jid': '20170221205153884009',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': '',
                    'diffs_file': '/home/regress/diff'}
            junos.rollback(**args)
            assert not mock_fopen.called

    def test_rollback_commit_check_exception(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not commit check due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.rollback(), ret)

    def test_rollback_commit_exception(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = True
            mock_commit.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = \
                'Rollback successful but commit failed with error "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.rollback(), ret)

    def test_rollback_commit_check_fails(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback:
            mock_commit_check.return_value = False
            ret = dict()
            ret['message'] = 'Rollback succesfull but pre-commit check failed.'
            ret['out'] = False
            self.assertEqual(junos.rollback(), ret)

    def test_diff_without_args(self):
        with patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
            junos.diff()
            mock_diff.assert_called_with(rb_id=0)

    def test_diff_with_arg(self):
        with patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
            junos.diff(id=2)
            mock_diff.assert_called_with(rb_id=2)

    def test_diff_exception(self):
        with patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
            mock_diff.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not get diff with error "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.diff(), ret)

    def test_ping_without_args(self):
        ret = dict()
        ret['message'] = 'Please specify the destination ip to ping.'
        ret['out'] = False
        self.assertEqual(junos.ping(), ret)

    def test_ping(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            junos.ping('1.1.1.1')
            args = mock_execute.call_args
            rpc = '<ping><count>5</count><host>1.1.1.1</host></ping>'
            self.assertEqualXML(args[0][0], rpc)

    def test_ping_ttl(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            args = {'__pub_user': 'sudo_drajvi',
                    '__pub_arg': ['1.1.1.1',
                                  {'ttl': 3}],
                    '__pub_fun': 'junos.ping',
                    '__pub_jid': '20170306165237683279',
                    '__pub_tgt': 'mac_min',
                    'ttl': 3,
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.ping('1.1.1.1', **args)
            exec_args = mock_execute.call_args
            rpc = '<ping><count>5</count><host>1.1.1.1</host><ttl>3</ttl></ping>'
            self.assertEqualXML(exec_args[0][0], rpc)

    def test_ping_exception(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            mock_execute.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Execution failed due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.ping('1.1.1.1'), ret)

    def test_cli_without_args(self):
        ret = dict()
        ret['message'] = 'Please provide the CLI command to be executed.'
        ret['out'] = False
        self.assertEqual(junos.cli(), ret)

    def test_cli_with_format_as_empty_string(self):
        with patch('jnpr.junos.device.Device.cli') as mock_cli:
            junos.cli('show version', format='')
            mock_cli.assert_called_with('show version', 'text', warning=False)

    def test_cli(self):
        with patch('jnpr.junos.device.Device.cli') as mock_cli:
            mock_cli.return_vale = 'CLI result'
            ret = dict()
            ret['message'] = 'CLI result'
            ret['out'] = True
            junos.cli('show version')
            mock_cli.assert_called_with('show version', 'text', warning=False)

    def test_cli_format_xml(self):
        with patch('salt.modules.junos.jxmlease.parse') as mock_jxml, \
                patch('salt.modules.junos.etree.tostring') as mock_to_string, \
                patch('jnpr.junos.device.Device.cli') as mock_cli:
            mock_cli.return_value = '<root><a>test</a></root>'
            mock_jxml.return_value = '<root><a>test</a></root>'
            args = {'__pub_user': 'root',
                    '__pub_arg': [{'format': 'xml'}],
                    'format': 'xml',
                    '__pub_fun': 'junos.cli',
                    '__pub_jid': '20170221182531323467',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            ret = dict()
            ret['message'] = '<root><a>test</a></root>'
            ret['out'] = True
            self.assertEqual(junos.cli('show version', **args), ret)
            mock_cli.assert_called_with('show version', 'xml', warning=False)
            mock_to_string.assert_called_once_with('<root><a>test</a></root>')
            assert mock_jxml.called

    def test_cli_exception_in_cli(self):
        with patch('jnpr.junos.device.Device.cli') as mock_cli:
            mock_cli.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Execution failed due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.cli('show version'), ret)

    def test_shutdown_without_args(self):
        ret = dict()
        ret['message'] = \
            'Provide either one of the arguments: shutdown or reboot.'
        ret['out'] = False
        self.assertEqual(junos.shutdown(), ret)

    def test_shutdown_with_reboot_args(self):
        with patch('salt.modules.junos.SW.reboot') as mock_reboot:
            ret = dict()
            ret['message'] = 'Successfully powered off/rebooted.'
            ret['out'] = True
            args = {'__pub_user': 'root', '__pub_arg': [{'reboot': True}],
                    'reboot': True, '__pub_fun': 'junos.shutdown',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}
            self.assertEqual(junos.shutdown(**args), ret)
            assert mock_reboot.called

    def test_shutdown_with_poweroff_args(self):
        with patch('salt.modules.junos.SW.poweroff') as mock_poweroff:
            ret = dict()
            ret['message'] = 'Successfully powered off/rebooted.'
            ret['out'] = True
            args = {'__pub_user': 'root', '__pub_arg': [{'shutdown': True}],
                    'reboot': True, '__pub_fun': 'junos.shutdown',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}
            self.assertEqual(junos.shutdown(**args), ret)
            assert mock_poweroff.called

    def test_shutdown_with_shutdown_as_false(self):
        ret = dict()
        ret['message'] = 'Nothing to be done.'
        ret['out'] = False
        args = {'__pub_user': 'root', '__pub_arg': [{'shutdown': False}],
                'reboot': True, '__pub_fun': 'junos.shutdown',
                '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                '__pub_tgt_type': 'glob', '__pub_ret': ''}
        self.assertEqual(junos.shutdown(**args), ret)

    def test_shutdown_with_in_min_arg(self):
        with patch('salt.modules.junos.SW.poweroff') as mock_poweroff:
            args = {'__pub_user': 'root',
                    'in_min': 10,
                    '__pub_arg': [{'in_min': 10,
                                   'shutdown': True}],
                    'reboot': True,
                    '__pub_fun': 'junos.shutdown',
                    '__pub_jid': '20170222231445709212',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.shutdown(**args)
            mock_poweroff.assert_called_with(in_min=10)

    def test_shutdown_with_at_arg(self):
        with patch('salt.modules.junos.SW.reboot') as mock_reboot:
            args = {'__pub_user': 'root',
                    '__pub_arg': [{'at': '12:00 pm',
                                   'reboot': True}],
                    'reboot': True,
                    '__pub_fun': 'junos.shutdown',
                    '__pub_jid': '201702276857',
                    'at': '12:00 pm',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.shutdown(**args)
            mock_reboot.assert_called_with(at='12:00 pm')

    def test_shutdown_fail_with_exception(self):
        with patch('salt.modules.junos.SW.poweroff') as mock_poweroff:
            mock_poweroff.side_effect = self.raise_exception
            args = {'__pub_user': 'root', '__pub_arg': [{'shutdown': True}],
                    'shutdown': True, '__pub_fun': 'junos.shutdown',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}
            ret = dict()
            ret['message'] = 'Could not poweroff/reboot beacause "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.shutdown(**args), ret)

    def test_install_config_without_args(self):
        ret = dict()
        ret['message'] = \
            'Please provide the salt path where the configuration is present'
        ret['out'] = False
        self.assertEqual(junos.install_config(), ret)

    def test_install_config_cp_fails(self):
        with patch('os.path.isfile') as mock_isfile:
            mock_isfile.return_value = False
            ret = dict()
            ret['message'] = 'Invalid file path.'
            ret['out'] = False
            self.assertEqual(junos.install_config('path'), ret)

    def test_install_config_file_cp_fails(self):
        with patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 0
            ret = dict()
            ret['message'] = 'Template failed to render'
            ret['out'] = False
            self.assertEqual(junos.install_config('path'), ret)

    def test_install_config(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(junos.install_config('actual/path/config.set'), ret)
            mock_load.assert_called_with(path='test/path/config', format='set')

    def test_install_config_xml_file(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(junos.install_config('actual/path/config.xml'), ret)
            mock_load.assert_called_with(path='test/path/config', format='xml')

    def test_install_config_text_file(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(junos.install_config('actual/path/config'), ret)
            mock_load.assert_called_with(path='test/path/config', format='text')

    def test_install_config_replace(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            args = {'__pub_user': 'root', '__pub_arg': [{'replace': True}],
                    'replace': True, '__pub_fun': 'junos.install_config',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(
                junos.install_config(
                    'actual/path/config.set',
                    **args),
                ret)
            mock_load.assert_called_with(
                path='test/path/config',
                format='set',
                merge=False)

    def test_install_config_overwrite(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            args = {'__pub_user': 'root', '__pub_arg': [{'overwrite': True}],
                    'overwrite': True, '__pub_fun': 'junos.install_config',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(
                junos.install_config(
                    'actual/path/config.xml',
                    **args),
                ret)
            mock_load.assert_called_with(
                path='test/path/config',
                format='xml',
                overwrite=True)

    def test_install_config_overwrite_false(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            args = {'__pub_user': 'root', '__pub_arg': [{'overwrite': False}],
                    'overwrite': False, '__pub_fun': 'junos.install_config',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(
                junos.install_config(
                    'actual/path/config',
                    **args),
                ret)
            mock_load.assert_called_with(
                path='test/path/config', format='text', merge=True)

    def test_install_config_load_causes_exception(self):
        with patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_load.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not load configuration due to : "Test exception"'
            ret['format'] = 'set'
            ret['out'] = False
            self.assertEqual(
                junos.install_config(
                    path='actual/path/config.set'), ret)

    def test_install_config_no_diff(self):
        with patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = None
            ret = dict()
            ret['message'] = 'Configuration already applied!'
            ret['out'] = True
            self.assertEqual(junos.install_config('actual/path/config'), ret)

    def test_install_config_write_diff(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('salt.utils.files.fopen') as mock_fopen, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True

            args = {'__pub_user': 'root',
                    '__pub_arg': [{'diffs_file': 'copy/config/here'}],
                    'diffs_file': 'copy/config/here',
                    '__pub_fun': 'junos.install_config',
                    '__pub_jid': '20170222213858582619',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}

            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(
                junos.install_config(
                    'actual/path/config',
                    **args),
                ret)
            mock_fopen.assert_called_with('copy/config/here', 'w')

    def test_install_config_write_diff_exception(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('salt.utils.files.fopen') as mock_fopen, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True
            mock_fopen.side_effect = self.raise_exception

            args = {'__pub_user': 'root',
                    '__pub_arg': [{'diffs_file': 'copy/config/here'}],
                    'diffs_file': 'copy/config/here',
                    '__pub_fun': 'junos.install_config',
                    '__pub_jid': '20170222213858582619',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}

            ret = dict()
            ret['message'] = 'Could not write into diffs_file due to: "Test exception"'
            ret['out'] = False
            self.assertEqual(
                junos.install_config(
                    'actual/path/config',
                    **args),
                ret)
            mock_fopen.assert_called_with('copy/config/here', 'w')

    def test_install_config_commit_params(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True
            args = {'comment': 'comitted via salt',
                    '__pub_user': 'root',
                    '__pub_arg': [{'comment': 'comitted via salt',
                                   'confirm': 3}],
                    'confirm': 3,
                    '__pub_fun': 'junos.commit',
                    '__pub_jid': '20170221182856987820',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            ret = dict()
            ret['message'] = 'Successfully loaded and committed!'
            ret['out'] = True
            self.assertEqual(
                junos.install_config(
                    'actual/path/config',
                    **args),
                ret)
            mock_commit.assert_called_with(comment='comitted via salt', confirm=3)

    def test_install_config_commit_check_exception(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.side_effect = self.raise_exception

            ret = dict()
            ret['message'] = 'Commit check threw the following exception: "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.install_config('actual/path/config.xml'), ret)

    def test_install_config_commit_check_fails(self):
        with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = False

            ret = dict()
            ret['message'] = 'Loaded configuration but commit check failed.'
            ret['out'] = False
            self.assertEqual(junos.install_config('actual/path/config.xml'), ret)

    def test_install_config_commit_exception(self):
        with patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                patch('jnpr.junos.utils.config.Config.load') as mock_load, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_isfile.return_value = True
            mock_getsize.return_value = 10
            mock_mkstemp.return_value = 'test/path/config'
            mock_diff.return_value = 'diff'
            mock_commit_check.return_value = True
            mock_commit.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = \
                'Commit check successful but commit failed with "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.install_config('actual/path/config'), ret)

    def test_zeroize(self):
        with patch('jnpr.junos.device.Device.cli') as mock_cli:
            result = junos.zeroize()
            ret = dict()
            ret['out'] = True
            ret['message'] = 'Completed zeroize and rebooted'
            mock_cli.assert_called_once_with('request system zeroize')
            self.assertEqual(result, ret)

    def test_zeroize_throw_exception(self):
        with patch('jnpr.junos.device.Device.cli') as mock_cli:
            mock_cli.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not zeroize due to : "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.zeroize(), ret)

    def test_install_os_without_args(self):
        ret = dict()
        ret['message'] = \
            'Please provide the salt path where the junos image is present.'
        ret['out'] = False
        self.assertEqual(junos.install_os(), ret)

    def test_install_os_cp_fails(self):
        with patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_getsize.return_value = 10
            mock_isfile.return_value = False
            ret = dict()
            ret['message'] = 'Invalid image path.'
            ret['out'] = False
            self.assertEqual(junos.install_os('/image/path/'), ret)

    def test_install_os_image_cp_fails(self):
        with patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_getsize.return_value = 0
            mock_isfile.return_value = True
            ret = dict()
            ret['message'] = 'Failed to copy image'
            ret['out'] = False
            self.assertEqual(junos.install_os('/image/path/'), ret)

    def test_install_os(self):
        with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_getsize.return_value = 10
            mock_isfile.return_value = True
            ret = dict()
            ret['out'] = True
            ret['message'] = 'Installed the os.'
            self.assertEqual(junos.install_os('path'), ret)

    def test_install_os_with_reboot_arg(self):
        with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                patch('jnpr.junos.utils.sw.SW.reboot') as mock_reboot, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_getsize.return_value = 10
            mock_isfile.return_value = True
            args = {'__pub_user': 'root', '__pub_arg': [{'reboot': True}],
                    'reboot': True, '__pub_fun': 'junos.install_os',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}
            ret = dict()
            ret['message'] = 'Successfully installed and rebooted!'
            ret['out'] = True
            self.assertEqual(junos.install_os('path', **args), ret)

    def test_install_os_pyez_install_throws_exception(self):
        with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_getsize.return_value = 10
            mock_isfile.return_value = True
            mock_install.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Installation failed due to: "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.install_os('path'), ret)

    def test_install_os_with_reboot_raises_exception(self):
        with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                patch('jnpr.junos.utils.sw.SW.reboot') as mock_reboot, \
                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                patch('os.path.isfile') as mock_isfile, \
                patch('os.path.getsize') as mock_getsize:
            mock_getsize.return_value = 10
            mock_isfile.return_value = True
            mock_reboot.side_effect = self.raise_exception
            args = {'__pub_user': 'root', '__pub_arg': [{'reboot': True}],
                    'reboot': True, '__pub_fun': 'junos.install_os',
                    '__pub_jid': '20170222213858582619', '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob', '__pub_ret': ''}
            ret = dict()
            ret['message'] = \
                'Installation successful but reboot failed due to : "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.install_os('path', **args), ret)

    def test_file_copy_without_args(self):
        ret = dict()
        ret['message'] = \
            'Please provide the absolute path of the file to be copied.'
        ret['out'] = False
        self.assertEqual(junos.file_copy(), ret)

    def test_file_copy_invalid_src(self):
        with patch('os.path.isfile') as mock_isfile:
            mock_isfile.return_value = False
            ret = dict()
            ret['message'] = 'Invalid source file path'
            ret['out'] = False
            self.assertEqual(junos.file_copy('invalid/file/path', 'file'), ret)

    def test_file_copy_without_dest(self):
        ret = dict()
        ret['message'] = \
            'Please provide the absolute path of the destination where the file is to be copied.'
        ret['out'] = False
        with patch('salt.modules.junos.os.path.isfile') as mck:
            mck.return_value = True
            self.assertEqual(junos.file_copy('/home/user/config.set'), ret)

    def test_file_copy(self):
        with patch('salt.modules.junos.SCP') as mock_scp, \
                patch('os.path.isfile') as mock_isfile:
            mock_isfile.return_value = True
            ret = dict()
            ret['message'] = 'Successfully copied file from test/src/file to file'
            ret['out'] = True
            self.assertEqual(
                junos.file_copy(
                    dest='file',
                    src='test/src/file'),
                ret)

    def test_file_copy_exception(self):
        with patch('salt.modules.junos.SCP') as mock_scp, \
                patch('os.path.isfile') as mock_isfile:
            mock_isfile.return_value = True
            mock_scp.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'Could not copy file : "Test exception"'
            ret['out'] = False
            self.assertEqual(
                junos.file_copy(
                    dest='file',
                    src='test/src/file'),
                ret)

    # These test cases test the __virtual__ function, used internally by salt
    # to check if the given module is loadable. This function is not used by
    # an external user.

    def test_virtual_proxy_unavailable(self):
        with patch.dict(junos.__opts__, {}):
            res = (False, 'The junos module could not be '
                          'loaded: junos-eznc or jxmlease or proxy could not be loaded.')
            self.assertEqual(junos.__virtual__(), res)

    def test_virtual_all_true(self):
        with patch.dict(junos.__opts__, {'proxy': 'test'}):
            self.assertEqual(junos.__virtual__(), 'junos')

    def test_rpc_without_args(self):
        ret = dict()
        ret['message'] = 'Please provide the rpc to execute.'
        ret['out'] = False
        self.assertEqual(junos.rpc(), ret)

    def test_rpc_get_config_exception(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            mock_execute.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'RPC execution failed due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.rpc('get_config'), ret)

    def test_rpc_get_config_filter(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            mock_execute.return_value = etree.XML('<reply><rpc/></reply>')
            args = {'__pub_user': 'root',
                    '__pub_arg': ['get-config',
                                  {'filter': '<configuration><system/></configuration>'}],
                    '__pub_fun': 'junos.rpc',
                    '__pub_jid': '20170314162715866528',
                    '__pub_tgt': 'mac_min',
                    '__pub_tgt_type': 'glob',
                    'filter': '<configuration><system/></configuration>',
                    '__pub_ret': ''}
            junos.rpc('get-config', **args)
            exec_args = mock_execute.call_args
            expected_rpc = '<get-configuration dev_timeout="30" ' \
                           'format="xml"><configuration><system/></configuration></get-configuration>'
            self.assertEqualXML(exec_args[0][0], expected_rpc)

    def test_rpc_get_interface_information(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            junos.rpc('get-interface-information', format='json')
            args = mock_execute.call_args
            expected_rpc = '<get-interface-information format="json"/>'
            self.assertEqualXML(args[0][0], expected_rpc)
            self.assertEqual(args[1], {'dev_timeout': 30})

    def test_rpc_get_interface_information_with_kwargs(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            args = {'__pub_user': 'root',
                    '__pub_arg': ['get-interface-information',
                                  '',
                                  'text',
                                  {'terse': True,
                                   'interface_name': 'lo0'}],
                    'terse': True,
                    '__pub_fun': 'junos.rpc',
                    '__pub_jid': '20170314160943363563',
                    '__pub_tgt': 'mac_min',
                    'interface_name': 'lo0',
                    '__pub_tgt_type': 'glob',
                    '__pub_ret': ''}
            junos.rpc('get-interface-information', format='text', **args)
            args = mock_execute.call_args
            expected_rpc = (
                    '<get-interface-information format="text">'
                    '<terse/><interface-name>lo0</interface-name></get-interface-information>'
            )
            self.assertEqualXML(etree.tostring(args[0][0]), expected_rpc)

    def test_rpc_get_chassis_inventory_filter_as_arg(self):
        with patch('salt.modules.junos.jxmlease.parse') as mock_jxmlease, \
                patch('salt.modules.junos.etree.tostring') as mock_tostring, \
                patch('salt.modules.junos.logging.Logger.warning') as mock_warning, \
                patch('jnpr.junos.device.Device.execute') as mock_execute:
            junos.rpc(
                'get-chassis-inventory',
                filter='<configuration><system/></configuration>')
            mock_warning.assert_called_with(
                'Filter ignored as it is only used with "get-config" rpc')

    def test_rpc_get_interface_information_exception(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            mock_execute.side_effect = self.raise_exception
            ret = dict()
            ret['message'] = 'RPC execution failed due to "Test exception"'
            ret['out'] = False
            self.assertEqual(junos.rpc('get_interface_information'), ret)

    def test_rpc_write_file_format_text(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute:
            mock_execute.return_value = etree.XML(
                '<rpc-reply>text rpc reply</rpc-reply>')
            with patch('salt.utils.files.fopen', mock_open(), create=True) as m_open:
                junos.rpc('get-chassis-inventory', '/path/to/file', format='text')
                writes = m_open.write_calls()
                assert writes == ['text rpc reply'], writes

    def test_rpc_write_file_format_json(self):
        with patch('jnpr.junos.device.Device.execute') as mock_execute, \
                patch('salt.utils.json.dumps') as mock_dumps:
            mock_dumps.return_value = 'json rpc reply'
            with patch('salt.utils.files.fopen', mock_open(), create=True) as m_open:
                junos.rpc('get-chassis-inventory', '/path/to/file', format='json')
                writes = m_open.write_calls()
                assert writes == ['json rpc reply'], writes

    def test_rpc_write_file(self):
        with patch('salt.modules.junos.jxmlease.parse') as mock_parse, \
                patch('salt.modules.junos.etree.tostring') as mock_tostring, \
                patch('jnpr.junos.device.Device.execute') as mock_execute:
            mock_tostring.return_value = 'xml rpc reply'
            with patch('salt.utils.files.fopen', mock_open(), create=True) as m_open:
                junos.rpc('get-chassis-inventory', '/path/to/file')
                writes = m_open.write_calls()
                assert writes == ['xml rpc reply'], writes



# -*- coding: utf-8 -*-
'''
Tests for the Chocolatey State
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest

# Import Salt libs
import salt.utils.platform

log = logging.getLogger(__name__)

__testcontext__ = {}


@destructiveTest
@skipIf(not salt.utils.platform.is_windows(), 'Windows Specific Test')
class ChocolateyTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Chocolatey State Tests
    These tests are destructive as the install and remove software
    '''

    def setUp(self):
        '''
        Ensure that Chocolatey is installed
        '''
        super(ChocolateyTest, self).setUp()
        if 'chocolatey' not in __testcontext__:
            self.run_function('chocolatey.bootstrap')
            __testcontext__['chocolatey'] = True

    def test_chocolatey(self):
        '''
        Test the following:
        - `chocolatey.installed`
        - `chocolatey.upgraded`
        - `chocolatey.uninstalled`
        '''
        # If this assert fails, we need to find new targets, this test needs to
        # be able to test successful installation of packages, so this package
        # needs to NOT be installed before we run the states below
        target = 'firefox'
        pre_version = '52.0.2'
        upg_version = '57.0.2'
        log.debug('Making sure %s is not installed', target)
        self.assertFalse(
            self.run_function('chocolatey.version', [target]))

        try:
            ####################################################
            # Test `chocolatey.installed`
            ####################################################
            # Install the package
            log.debug('Testing chocolatey.installed')
            ret = self.run_state(
                'chocolatey.installed',
                name=target,
                version=pre_version)
            self.assertSaltTrueReturn(ret)

            # Verify the package is installed
            log.debug('Verifying install success')
            ret = self.run_function('chocolatey.version', [target])
            self.assertEqual(ret, {'Firefox': [pre_version]})

            ####################################################
            # Test `chocolatey.upgraded`
            ####################################################
            # Upgrade the package
            log.debug('Testing chocolatey.upgraded')
            ret = self.run_state(
                'chocolatey.upgraded',
                name=target,
                version=upg_version)
            self.assertSaltTrueReturn(ret)

            # Verify the package is upgraded
            log.debug('Verifying upgrade success')
            ret = self.run_function('chocolatey.version', [target])
            self.assertEqual(ret, {'Firefox': [upg_version]})

            ####################################################
            # Test `chocolatey.uninstalled`
            ####################################################
            # uninstall the package
            log.debug('Testing chocolatey.uninstalled')
            ret = self.run_state('chocolatey.uninstalled', name=target)
            self.assertSaltTrueReturn(ret)

            # Verify the package is uninstalled
            log.debug('Verifying uninstall success')
            ret = self.run_function('chocolatey.version', [target])
            self.assertEqual(ret, {})

        finally:
            # Always uninstall
            log.debug('Uninstalling %s', target)
            self.run_function('chocolatey.uninstall', [target])



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.ps as ps

PATCH_OPTS = dict(autospec=True, spec_set=True)


class FakeProcess(object):

    def __init__(self, _name, pid):
        self._name = _name
        self.pid = pid

    def name(self):
        return self._name


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PSBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.[s]
    '''

    def setup_loader_modules(self):
        return {}

    def test_non_list_config(self):
        config = {}

        ret = ps.validate(config)

        self.assertEqual(ret, (False, 'Configuration for ps beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = ps.validate(config)

        self.assertEqual(ret, (False, 'Configuration for ps '
                                      'beacon requires processes.'))

    def test_ps_running(self):
        with patch('salt.utils.psutil_compat.process_iter', **PATCH_OPTS) as mock_process_iter:
            mock_process_iter.return_value = [FakeProcess(_name='salt-master', pid=3),
                                              FakeProcess(_name='salt-minion', pid=4)]
            config = [{'processes': {'salt-master': 'running'}}]

            ret = ps.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = ps.beacon(config)
            self.assertEqual(ret, [{'salt-master': 'Running'}])

    def test_ps_not_running(self):
        with patch('salt.utils.psutil_compat.process_iter', **PATCH_OPTS) as mock_process_iter:
            mock_process_iter.return_value = [FakeProcess(_name='salt-master', pid=3),
                                              FakeProcess(_name='salt-minion', pid=4)]
            config = [{'processes': {'mysql': 'stopped'}}]

            ret = ps.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = ps.beacon(config)
            self.assertEqual(ret, [{'mysql': 'Stopped'}])



# -*- coding: utf-8 -*-
'''
    :codeauthor: Eric Radman <ericshane@eradman.com>
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON,
)

# Import Salt libs
import salt.cloud

EXAMPLE_PROVIDERS = {
 'nyc_vcenter': {'vmware': {'driver': 'vmware',
                            'password': '123456',
                            'profiles': {'nyc-vm': {'cluster': 'nycvirt',
                                                    'datastore': 'datastore1',
                                                    'devices': {'disk': {'Hard disk 1': {'controller': 'SCSI controller 1',
                                                                                         'size': 20}},
                                                                 'network': {'Network Adapter 1': {'mac': '44:44:44:44:44:42',
                                                                                                   'name': 'vlan50',
                                                                                                   'switch_type': 'standard'}},
                                                                 'scsi': {'SCSI controller 1': {'type': 'paravirtual'}}},
                                                    'extra_config': {'mem.hotadd': 'yes'},
                                                    'folder': 'coreinfra',
                                                    'image': 'rhel6_64Guest',
                                                    'memory': '8GB',
                                                    'num_cpus': 2,
                                                    'power_on': True,
                                                    'profile': 'nyc-vm',
                                                    'provider': 'nyc_vcenter:vmware',
                                                    'resourcepool': 'Resources'}},
                            'url': 'vca1.saltstack.com',
                            'user': 'root'}}
}

EXAMPLE_PROFILES = {
 'nyc-vm': {'cluster': 'nycvirt',
            'datastore': 'datastore1',
            'devices': {'disk': {'Hard disk 1': {'controller': 'SCSI controller 1',
                                                 'size': 20}},
                        'network': {'Network Adapter 1': {'mac': '44:44:44:44:44:42',
                                                          'name': 'vlan50',
                                                          'switch_type': 'standard'}},
                        'scsi': {'SCSI controller 1': {'type': 'paravirtual'}}},
            'extra_config': {'mem.hotadd': 'yes'},
            'folder': 'coreinfra',
            'image': 'rhel6_64Guest',
            'memory': '8GB',
            'num_cpus': 2,
            'power_on': True,
            'profile': 'nyc-vm',
            'provider': 'nyc_vcenter:vmware',
            'resourcepool': 'Resources'}
}

EXAMPLE_MAP = {
 'nyc-vm': {'db1': {'cpus': 4,
                    'devices': {'disk': {'Hard disk 1': {'size': 40}},
                                'network': {'Network Adapter 1': {'mac': '22:4a:b2:92:b3:eb'}}},
                    'memory': '16GB',
                    'name': 'db1'}}
}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MapConfTest(TestCase):
    '''
    Validate evaluation of salt-cloud map configuration
    '''

    def test_cloud_map_merge_conf(self):
        '''
        Ensure that nested values can be selectivly overridden in a map file
        '''
        with patch('salt.config.check_driver_dependencies', MagicMock(return_value=True)), \
                patch('salt.cloud.Map.read', MagicMock(return_value=EXAMPLE_MAP)):
            self.maxDiff = None
            opts = {'extension_modules': '/var/cache/salt/master/extmods',
                    'optimization_order': [0, 1, 2],
                    'providers': EXAMPLE_PROVIDERS, 'profiles': EXAMPLE_PROFILES}
            cloud_map = salt.cloud.Map(opts)
            merged_profile = {
             'create': {'db1': {'cluster': 'nycvirt',
                                'cpus': 4,
                                'datastore': 'datastore1',
                                'devices': {'disk': {'Hard disk 1': {'controller': 'SCSI controller 1',
                                                                     'size': 40}},
                                            'network': {'Network Adapter 1': {'mac': '22:4a:b2:92:b3:eb',
                                                                              'name': 'vlan50',
                                                                              'switch_type': 'standard'}},
                                            'scsi': {'SCSI controller 1': {'type': 'paravirtual'}}},
                                'driver': 'vmware',
                                'extra_config': {'mem.hotadd': 'yes'},
                                'folder': 'coreinfra',
                                'image': 'rhel6_64Guest',
                                'memory': '16GB',
                                'name': 'db1',
                                'num_cpus': 2,
                                'password': '123456',
                                'power_on': True,
                                'profile': 'nyc-vm',
                                'provider': 'nyc_vcenter:vmware',
                                'resourcepool': 'Resources',
                                'url': 'vca1.saltstack.com',
                                'user': 'root'}}
            }
            self.assertEqual(cloud_map.map_data(), merged_profile)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch
)

# Import Salt Libs
import salt.modules.win_shadow as win_shadow


class WinShadowTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_shadow
    '''
    def setup_loader_modules(self):
        return {
            win_shadow: {
                '__salt__': {
#                    'user.info': MagicMock(return_value=True),
                    'user.update': MagicMock(return_value=True)
                }
            }
        }

    # 'info' function tests: 1

    def test_info(self):
        '''
        Test if it return information for the specified user
        '''
        mock_user_info = MagicMock(return_value={'name': 'SALT',
                                                 'password_changed': '',
                                                 'expiration_date': ''})
        with patch.dict(win_shadow.__salt__, {'user.info': mock_user_info}):
            self.assertDictEqual(win_shadow.info('SALT'), {'name': 'SALT',
                                                           'passwd': 'Unavailable',
                                                           'lstchg': '',
                                                           'min': '',
                                                           'max': '',
                                                           'warn': '',
                                                           'inact': '',
                                                           'expire': ''})

    # 'set_password' function tests: 1

    def test_set_password(self):
        '''
        Test if it set the password for a named user.
        '''
        mock_cmd = MagicMock(return_value={'retcode': False})
        mock_user_info = MagicMock(return_value={'name': 'SALT',
                                                 'password_changed': '',
                                                 'expiration_date': ''})
        with patch.dict(win_shadow.__salt__, {'cmd.run_all': mock_cmd,
                                              'user.info': mock_user_info}):
            self.assertTrue(win_shadow.set_password('root', 'mysecretpassword'))



# -*- coding: utf-8 -*-
'''
Unit Tests for functions located in salt/utils/files.py
'''

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function
import copy
import os

# Import Salt libs
import salt.utils.files
from salt.ext import six

# Import Salt Testing libs
from tests.support.helpers import with_tempdir
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)


class FilesTestCase(TestCase):
    '''
    Test case for files util.
    '''

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_safe_rm(self):
        with patch('os.remove') as os_remove_mock:
            salt.utils.files.safe_rm('dummy_tgt')
            self.assertTrue(os_remove_mock.called)

    @skipIf(os.path.exists('/tmp/no_way_this_is_a_file_nope.sh'), 'Test file exists! Skipping safe_rm_exceptions test!')
    def test_safe_rm_exceptions(self):
        error = False
        try:
            salt.utils.files.safe_rm('/tmp/no_way_this_is_a_file_nope.sh')
        except (IOError, OSError):
            error = True
        self.assertFalse(error, 'salt.utils.files.safe_rm raised exception when it should not have')

    @with_tempdir()
    def test_safe_walk_symlink_recursion(self, tmp):
        if os.stat(tmp).st_ino == 0:
            self.skipTest('inodes not supported in {0}'.format(tmp))
        os.mkdir(os.path.join(tmp, 'fax'))
        os.makedirs(os.path.join(tmp, 'foo', 'bar'))
        os.symlink(os.path.join('..', '..'), os.path.join(tmp, 'foo', 'bar', 'baz'))
        os.symlink('foo', os.path.join(tmp, 'root'))
        expected = [
            (os.path.join(tmp, 'root'), ['bar'], []),
            (os.path.join(tmp, 'root', 'bar'), ['baz'], []),
            (os.path.join(tmp, 'root', 'bar', 'baz'), ['fax', 'foo', 'root'], []),
            (os.path.join(tmp, 'root', 'bar', 'baz', 'fax'), [], []),
        ]
        paths = []
        for root, dirs, names in salt.utils.files.safe_walk(os.path.join(tmp, 'root')):
            paths.append((root, sorted(dirs), names))
        if paths != expected:
            raise AssertionError(
                '\n'.join(
                    ['got:'] + [repr(p) for p in paths] +
                    ['', 'expected:'] + [repr(p) for p in expected]
                )
            )

    @skipIf(not six.PY3, 'This test only applies to Python 3')
    def test_fopen_with_disallowed_fds(self):
        '''
        This is safe to have as a unit test since we aren't going to actually
        try to read or write. We want to ensure that we are raising a
        TypeError. Python 3's open() builtin will treat the booleans as file
        descriptor numbers and try to open stdin/stdout. We also want to test
        fd 2 which is stderr.
        '''
        for invalid_fn in (False, True, 0, 1, 2):
            try:
                with salt.utils.files.fopen(invalid_fn):
                    pass
            except TypeError:
                # This is expected. We aren't using an assertRaises here
                # because we want to ensure that if we did somehow open the
                # filehandle, that it doesn't remain open.
                pass
            else:
                # We probably won't even get this far if we actually opened
                # stdin/stdout as a file descriptor. It is likely to cause the
                # integration suite to die since, news flash, closing
                # stdin/stdout/stderr is usually not a wise thing to do in the
                # middle of a program's execution.
                self.fail(
                    'fopen() should have been prevented from opening a file '
                    'using {0} as the filename'.format(invalid_fn)
                )

    def _create_temp_structure(self, temp_directory, structure):
        for folder, files in six.iteritems(structure):
            current_directory = os.path.join(temp_directory, folder)
            os.makedirs(current_directory)
            for name, content in six.iteritems(files):
                path = os.path.join(temp_directory, folder, name)
                with salt.utils.files.fopen(path, 'w+') as fh:
                    fh.write(content)

    def _validate_folder_structure_and_contents(self, target_directory,
                                                desired_structure):
        for folder, files in six.iteritems(desired_structure):
            for name, content in six.iteritems(files):
                path = os.path.join(target_directory, folder, name)
                with salt.utils.files.fopen(path) as fh:
                    assert fh.read().strip() == content

    @with_tempdir()
    @with_tempdir()
    def test_recursive_copy(self, src, dest):
        src_structure = {
            'foo': {
                'foofile.txt': 'fooSTRUCTURE'
            },
            'bar': {
                'barfile.txt': 'barSTRUCTURE'
            }
        }
        dest_structure = {
            'foo': {
                'foo.txt': 'fooTARGET_STRUCTURE'
            },
            'baz': {
                'baz.txt': 'bazTARGET_STRUCTURE'
            }
        }

        # Create the file structures in both src and dest dirs
        self._create_temp_structure(src, src_structure)
        self._create_temp_structure(dest, dest_structure)

        # Perform the recursive copy
        salt.utils.files.recursive_copy(src, dest)

        # Confirm results match expected results
        desired_structure = copy.copy(dest_structure)
        desired_structure.update(src_structure)
        self._validate_folder_structure_and_contents(
            dest,
            desired_structure)



# -*- coding: utf-8 -*-
'''
Integration tests for the docker_container states
'''
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import errno
import functools
import logging
import os
import subprocess
import tempfile

# Import Salt Testing Libs
from tests.support.unit import skipIf
from tests.support.case import ModuleCase
from tests.support.docker import with_network, random_name
from tests.support.paths import FILES, TMP
from tests.support.helpers import destructiveTest, with_tempdir
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Libs
import salt.utils.files
import salt.utils.network
import salt.utils.path
from salt.exceptions import CommandExecutionError

# Import 3rd-party libs
from salt.ext import six

log = logging.getLogger(__name__)

IPV6_ENABLED = bool(salt.utils.network.ip_addrs6(include_loopback=True))


def container_name(func):
    '''
    Generate a randomized name for a container and clean it up afterward
    '''
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        name = random_name(prefix='salt_test_')
        try:
            return func(self, name, *args, **kwargs)
        finally:
            try:
                self.run_function('docker.rm', [name], force=True)
            except CommandExecutionError as exc:
                if 'No such container' not in exc.__str__():
                    raise
    return wrapper


@destructiveTest
@skipIf(not salt.utils.path.which('busybox'), 'Busybox not installed')
@skipIf(not salt.utils.path.which('dockerd'), 'Docker not installed')
class DockerContainerTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Test docker_container states
    '''
    @classmethod
    def setUpClass(cls):
        '''
        '''
        # Create temp dir
        cls.image_build_rootdir = tempfile.mkdtemp(dir=TMP)
        # Generate image name
        cls.image = random_name(prefix='salt_busybox_')

        script_path = \
            os.path.join(FILES, 'file/base/mkimage-busybox-static')
        cmd = [script_path, cls.image_build_rootdir, cls.image]
        log.debug('Running \'%s\' to build busybox image', ' '.join(cmd))
        process = subprocess.Popen(
            cmd,
            close_fds=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT)
        output = process.communicate()[0]
        log.debug('Output from mkimge-busybox-static:\n%s', output)

        if process.returncode != 0:
            raise Exception('Failed to build image')

        try:
            salt.utils.files.rm_rf(cls.image_build_rootdir)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise

    @classmethod
    def tearDownClass(cls):
        cmd = ['docker', 'rmi', '--force', cls.image]
        log.debug('Running \'%s\' to destroy busybox image', ' '.join(cmd))
        process = subprocess.Popen(
            cmd,
            close_fds=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT)
        output = process.communicate()[0]
        log.debug('Output from %s:\n%s', ' '.join(cmd), output)

        if process.returncode != 0:
            raise Exception('Failed to destroy image')

    def run_state(self, function, **kwargs):
        ret = super(DockerContainerTestCase, self).run_state(function, **kwargs)
        log.debug('ret = %s', ret)
        return ret

    @with_tempdir()
    @container_name
    def test_running_with_no_predefined_volume(self, name, bind_dir_host):
        '''
        This tests that a container created using the docker_container.running
        state, with binds defined, will also create the corresponding volumes
        if they aren't pre-defined in the image.
        '''
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            binds=bind_dir_host + ':/foo',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Now check to ensure that the container has volumes to match the
        # binds that we used when creating it.
        ret = self.run_function('docker.inspect_container', [name])
        self.assertTrue('/foo' in ret['Config']['Volumes'])

    @container_name
    def test_running_with_no_predefined_ports(self, name):
        '''
        This tests that a container created using the docker_container.running
        state, with port_bindings defined, will also configure the
        corresponding ports if they aren't pre-defined in the image.
        '''
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            port_bindings='14505-14506:24505-24506,2123:2123/udp,8080',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Now check to ensure that the container has ports to match the
        # port_bindings that we used when creating it.
        expected_ports = (4505, 4506, 8080, '2123/udp')
        ret = self.run_function('docker.inspect_container', [name])
        self.assertTrue(x in ret['NetworkSettings']['Ports']
                        for x in expected_ports)

    @container_name
    def test_running_updated_image_id(self, name):
        '''
        This tests the case of an image being changed after the container is
        created. The next time the state is run, the container should be
        replaced because the image ID is now different.
        '''
        # Create and start a container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Get the container's info
        c_info = self.run_function('docker.inspect_container', [name])
        c_name, c_id = (c_info[x] for x in ('Name', 'Id'))
        # Alter the filesystem inside the container
        self.assertEqual(
            self.run_function('docker.retcode', [name, 'touch /.salttest']),
            0
        )
        # Commit the changes and overwrite the test class' image
        self.run_function('docker.commit', [c_id, self.image])
        # Re-run the state
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was replaced
        self.assertTrue('container_id' in ret['changes'])
        # Check to make sure that the image is in the changes dict, since
        # it should have changed
        self.assertTrue('image' in ret['changes'])
        # Check that the comment in the state return states that
        # container's image has changed
        self.assertTrue('Container has a new image' in ret['comment'])

    @container_name
    def test_running_start_false_without_replace(self, name):
        '''
        Test that we do not start a container which is stopped, when it is not
        being replaced.
        '''
        # Create a container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Stop the container
        self.run_function('docker.stop', [name], force=True)
        # Re-run the state with start=False
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            start=False,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was not replaced
        self.assertTrue('container_id' not in ret['changes'])
        # Check to make sure that the state is not the changes dict, since
        # it should not have changed
        self.assertTrue('state' not in ret['changes'])

    @container_name
    def test_running_start_false_with_replace(self, name):
        '''
        Test that we do start a container which was previously stopped, even
        though start=False, because the container was replaced.
        '''
        # Create a container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Stop the container
        self.run_function('docker.stop', [name], force=True)
        # Re-run the state with start=False but also change the command to
        # trigger the container being replaced.
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            command='sleep 600',
            start=False,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was not replaced
        self.assertTrue('container_id' in ret['changes'])
        # Check to make sure that the state is not the changes dict, since
        # it should not have changed
        self.assertTrue('state' not in ret['changes'])

    @container_name
    def test_running_start_true(self, name):
        '''
        This tests that we *do* start a container that is stopped, when the
        "start" argument is set to True.
        '''
        # Create a container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Stop the container
        self.run_function('docker.stop', [name], force=True)
        # Re-run the state with start=True
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            start=True,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was not replaced
        self.assertTrue('container_id' not in ret['changes'])
        # Check to make sure that the state is in the changes dict, since
        # it should have changed
        self.assertTrue('state' in ret['changes'])
        # Check that the comment in the state return states that
        # container's state has changed
        self.assertTrue(
            "State changed from 'stopped' to 'running'" in ret['comment'])

    @container_name
    def test_running_with_invalid_input(self, name):
        '''
        This tests that the input tranlation code identifies invalid input and
        includes information about that invalid argument in the state return.
        '''
        # Try to create a container with invalid input
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            ulimits='nofile:2048',
            shutdown_timeout=1,
        )
        self.assertSaltFalseReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was not created
        self.assertTrue('container_id' not in ret['changes'])
        # Check that the error message about the invalid argument is
        # included in the comment for the state
        self.assertTrue(
            'Ulimit definition \'nofile:2048\' is not in the format '
            'type=soft_limit[:hard_limit]' in ret['comment']
        )

    @container_name
    def test_running_with_argument_collision(self, name):
        '''
        this tests that the input tranlation code identifies an argument
        collision (API args and their aliases being simultaneously used) and
        includes information about them in the state return.
        '''
        # try to create a container with invalid input
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            ulimits='nofile=2048',
            ulimit='nofile=1024:2048',
            shutdown_timeout=1,
        )
        self.assertSaltFalseReturn(ret)
        # Ciscard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was not created
        self.assertTrue('container_id' not in ret['changes'])
        # Check that the error message about the collision is included in
        # the comment for the state
        self.assertTrue(
            '\'ulimit\' is an alias for \'ulimits\'' in ret['comment'])

    @container_name
    def test_running_with_ignore_collisions(self, name):
        '''
        This tests that the input tranlation code identifies an argument
        collision (API args and their aliases being simultaneously used)
        includes information about them in the state return.
        '''
        # try to create a container with invalid input
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            ignore_collisions=True,
            ulimits='nofile=2048',
            ulimit='nofile=1024:2048',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check to make sure that the container was created
        self.assertTrue('container_id' in ret['changes'])
        # Check that the value from the API argument was one that was used
        # to create the container
        c_info = self.run_function('docker.inspect_container', [name])
        actual = c_info['HostConfig']['Ulimits']
        expected = [{'Name': 'nofile', 'Soft': 2048, 'Hard': 2048}]
        self.assertEqual(actual, expected)

    @container_name
    def test_running_with_removed_argument(self, name):
        '''
        This tests that removing an argument from a created container will
        be detected and result in the container being replaced.

        It also tests that we revert back to the value from the image. This
        way, when the "command" argument is removed, we confirm that we are
        reverting back to the image's command.
        '''
        # Create the container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            command='sleep 600',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Run the state again with the "command" argument removed
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Now check to ensure that the changes include the command
        # reverting back to the image's command.
        image_info = self.run_function('docker.inspect_image', [self.image])
        self.assertEqual(
            ret['changes']['container']['Config']['Cmd']['new'],
            image_info['Config']['Cmd']
        )

    @container_name
    def test_running_with_port_bindings(self, name):
        '''
        This tests that the ports which are being bound are also exposed, even
        when not explicitly configured. This test will create a container with
        only some of the ports exposed, including some which aren't even bound.
        The resulting containers exposed ports should contain all of the ports
        defined in the "ports" argument, as well as each of the ports which are
        being bound.
        '''
        # Create the container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            command='sleep 600',
            shutdown_timeout=1,
            port_bindings=[1234, '1235-1236', '2234/udp', '2235-2236/udp'],
            ports=[1235, '2235/udp', 9999],
        )
        self.assertSaltTrueReturn(ret)

        # Check the created container's port bindings and exposed ports. The
        # port bindings should only contain the ports defined in the
        # port_bindings argument, while the exposed ports should also contain
        # the extra port (9999/tcp) which was included in the ports argument.
        cinfo = self.run_function('docker.inspect_container', [name])
        ports = ['1234/tcp', '1235/tcp', '1236/tcp',
                 '2234/udp', '2235/udp', '2236/udp']
        self.assertEqual(
            sorted(cinfo['HostConfig']['PortBindings']),
            ports
        )
        self.assertEqual(
            sorted(cinfo['Config']['ExposedPorts']),
            ports + ['9999/tcp']
        )

    @container_name
    def test_absent_with_stopped_container(self, name):
        '''
        This tests the docker_container.absent state on a stopped container
        '''
        # Create the container
        self.run_function('docker.create', [self.image], name=name)
        # Remove the container
        ret = self.run_state(
            'docker_container.absent',
            name=name,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check that we have a removed container ID in the changes dict
        self.assertTrue('removed' in ret['changes'])

        # Run the state again to confirm it changes nothing
        ret = self.run_state(
            'docker_container.absent',
            name=name,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Nothing should have changed
        self.assertEqual(ret['changes'], {})
        # Ensure that the comment field says the container does not exist
        self.assertEqual(
            ret['comment'],
            'Container \'{0}\' does not exist'.format(name)
        )

    @container_name
    def test_absent_with_running_container(self, name):
        '''
        This tests the docker_container.absent state and
        '''
        # Create the container
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            command='sleep 600',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)

        # Try to remove the container. This should fail because force=True
        # is needed to remove a container that is running.
        ret = self.run_state(
            'docker_container.absent',
            name=name,
            shutdown_timeout=1,
        )
        self.assertSaltFalseReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Nothing should have changed
        self.assertEqual(ret['changes'], {})
        # Ensure that the comment states that force=True is required
        self.assertEqual(
            ret['comment'],
            'Container is running, set force to True to forcibly remove it'
        )

        # Try again with force=True. This should succeed.
        ret = self.run_state('docker_container.absent',
            name=name,
            force=True,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        # Discard the outer dict with the state compiler data to make below
        # asserts easier to read/write
        ret = ret[next(iter(ret))]
        # Check that we have a removed container ID in the changes dict
        self.assertTrue('removed' in ret['changes'])
        # The comment should mention that the container was removed
        self.assertEqual(
            ret['comment'],
            'Forcibly removed container \'{0}\''.format(name)
        )

    @container_name
    def test_running_image_name(self, name):
        '''
        Ensure that we create the container using the image name instead of ID
        '''
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_function('docker.inspect_container', [name])
        self.assertEqual(ret['Config']['Image'], self.image)

    @container_name
    def test_env_with_running_container(self, name):
        '''
        docker_container.running environnment part. Testing issue 39838.
        '''
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            env='VAR1=value1,VAR2=value2,VAR3=value3',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_function('docker.inspect_container', [name])
        self.assertTrue('VAR1=value1' in ret['Config']['Env'])
        self.assertTrue('VAR2=value2' in ret['Config']['Env'])
        self.assertTrue('VAR3=value3' in ret['Config']['Env'])
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image,
            env='VAR1=value1,VAR2=value2',
            shutdown_timeout=1,
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_function('docker.inspect_container', [name])
        self.assertTrue('VAR1=value1' in ret['Config']['Env'])
        self.assertTrue('VAR2=value2' in ret['Config']['Env'])
        self.assertTrue('VAR3=value3' not in ret['Config']['Env'])

    def _test_running(self, container_name, *nets):
        '''
        DRY function for testing static IPs
        '''
        networks = []
        for net in nets:
            net_def = {
                net.name: [
                    {net.ip_arg: net[0]}
                ]
            }
            networks.append(net_def)

        kwargs = {
            'name': container_name,
            'image': self.image,
            'networks': networks,
            'shutdown_timeout': 1,
        }
        # Create a container
        ret = self.run_state('docker_container.running', **kwargs)
        self.assertSaltTrueReturn(ret)

        inspect_result = self.run_function('docker.inspect_container',
                                           [container_name])
        connected_networks = inspect_result['NetworkSettings']['Networks']

        # Check that the correct IP was set
        try:
            for net in nets:
                self.assertEqual(
                    connected_networks[net.name]['IPAMConfig'][net.arg_map(net.ip_arg)],
                    net[0]
                )
        except KeyError:
            # Fail with a meaningful error
            msg = (
                'Container does not have the expected network config for '
                'network {0}'.format(net.name)
            )
            log.error(msg)
            log.error('Connected networks: %s', connected_networks)
            self.fail('{0}. See log for more information.'.format(msg))

        # Check that container continued running and didn't immediately exit
        self.assertTrue(inspect_result['State']['Running'])

        # Update the SLS configuration to use the second random IP so that we
        # can test updating a container's network configuration without
        # replacing the container.
        for idx, net in enumerate(nets):
            kwargs['networks'][idx][net.name][0][net.ip_arg] = net[1]
        ret = self.run_state('docker_container.running', **kwargs)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        expected = {'container': {'Networks': {}}}
        for net in nets:
            expected['container']['Networks'][net.name] = {
                'IPAMConfig': {
                    'old': {net.arg_map(net.ip_arg): net[0]},
                    'new': {net.arg_map(net.ip_arg): net[1]},
                }
            }
        self.assertEqual(ret['changes'], expected)

        expected = [
            "Container '{0}' is already configured as specified.".format(
                container_name
            )
        ]
        expected.extend([
            "Reconnected to network '{0}' with updated configuration.".format(
                x.name
            )
            for x in sorted(nets, key=lambda y: y.name)
        ])
        expected = ' '.join(expected)
        self.assertEqual(ret['comment'], expected)

        # Update the SLS configuration to remove the last network
        log.critical('networks = %s', kwargs['networks'])
        kwargs['networks'].pop(-1)
        ret = self.run_state('docker_container.running', **kwargs)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        expected = {
            'container': {
                'Networks': {
                    nets[-1].name: {
                        'IPAMConfig': {
                            'old': {
                                nets[-1].arg_map(nets[-1].ip_arg): nets[-1][1]
                            },
                            'new': None,
                        }
                    }
                }
            }
        }
        self.assertEqual(ret['changes'], expected)

        expected = (
            "Container '{0}' is already configured as specified. Disconnected "
            "from network '{1}'.".format(container_name, nets[-1].name)
        )
        self.assertEqual(ret['comment'], expected)

        # Update the SLS configuration to add back the last network, only use
        # an automatic IP instead of static IP.
        kwargs['networks'].append(nets[-1].name)
        ret = self.run_state('docker_container.running', **kwargs)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Get the automatic IP by inspecting the container, and use it to build
        # the expected changes.
        container_netinfo = self.run_function(
            'docker.inspect_container',
            [container_name]).get('NetworkSettings', {}).get('Networks', {})[nets[-1].name]
        autoip_keys = self.minion_opts['docker.compare_container_networks']['automatic']
        autoip_config = {
            x: y for x, y in six.iteritems(container_netinfo)
            if x in autoip_keys and y
        }

        expected = {'container': {'Networks': {nets[-1].name: {}}}}
        for key, val in six.iteritems(autoip_config):
            expected['container']['Networks'][nets[-1].name][key] = {
                'old': None, 'new': val
            }
        self.assertEqual(ret['changes'], expected)

        expected = (
            "Container '{0}' is already configured as specified. Connected "
            "to network '{1}'.".format(container_name, nets[-1].name)
        )
        self.assertEqual(ret['comment'], expected)

        # Update the SLS configuration to remove the last network
        kwargs['networks'].pop(-1)
        ret = self.run_state('docker_container.running', **kwargs)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        expected = {'container': {'Networks': {nets[-1].name: {}}}}
        for key, val in six.iteritems(autoip_config):
            expected['container']['Networks'][nets[-1].name][key] = {
                'old': val, 'new': None
            }
        self.assertEqual(ret['changes'], expected)

        expected = (
            "Container '{0}' is already configured as specified. Disconnected "
            "from network '{1}'.".format(container_name, nets[-1].name)
        )
        self.assertEqual(ret['comment'], expected)

    @with_network(subnet='10.247.197.96/27', create=True)
    @container_name
    def test_running_ipv4(self, container_name, *nets):
        self._test_running(container_name, *nets)

    @with_network(subnet='10.247.197.128/27', create=True)
    @with_network(subnet='10.247.197.96/27', create=True)
    @container_name
    def test_running_dual_ipv4(self, container_name, *nets):
        self._test_running(container_name, *nets)

    @with_network(subnet='fe3f:2180:26:1::/123', create=True)
    @container_name
    @skipIf(not IPV6_ENABLED, 'IPv6 not enabled')
    def test_running_ipv6(self, container_name, *nets):
        self._test_running(container_name, *nets)

    @with_network(subnet='fe3f:2180:26:1::20/123', create=True)
    @with_network(subnet='fe3f:2180:26:1::/123', create=True)
    @container_name
    @skipIf(not IPV6_ENABLED, 'IPv6 not enabled')
    def test_running_dual_ipv6(self, container_name, *nets):
        self._test_running(container_name, *nets)

    @with_network(subnet='fe3f:2180:26:1::/123', create=True)
    @with_network(subnet='10.247.197.96/27', create=True)
    @container_name
    @skipIf(not IPV6_ENABLED, 'IPv6 not enabled')
    def test_running_mixed_ipv4_and_ipv6(self, container_name, *nets):
        self._test_running(container_name, *nets)

    @container_name
    def test_run_with_onlyif(self, name):
        '''
        Test docker_container.run with onlyif. The container should not run
        (and the state should return a True result) if the onlyif has a nonzero
        return code, but if the onlyif has a zero return code the container
        should run.
        '''
        for cmd in ('/bin/false', ['/bin/true', '/bin/false']):
            log.debug('Trying %s', cmd)
            ret = self.run_state(
                'docker_container.run',
                name=name,
                image=self.image,
                command='whoami',
                onlyif=cmd)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertFalse(ret['changes'])
            self.assertTrue(
                ret['comment'].startswith(
                    'onlyif command /bin/false returned exit code of'
                )
            )
            self.run_function('docker.rm', [name], force=True)

        for cmd in ('/bin/true', ['/bin/true', 'ls /']):
            log.debug('Trying %s', cmd)
            ret = self.run_state(
                'docker_container.run',
                name=name,
                image=self.image,
                command='whoami',
                onlyif=cmd)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertEqual(ret['changes']['Logs'], 'root\n')
            self.assertEqual(
                ret['comment'],
                'Container ran and exited with a return code of 0'
            )
            self.run_function('docker.rm', [name], force=True)

    @container_name
    def test_run_with_unless(self, name):
        '''
        Test docker_container.run with unless. The container should not run
        (and the state should return a True result) if the unless has a zero
        return code, but if the unless has a nonzero return code the container
        should run.
        '''
        for cmd in ('/bin/true', ['/bin/false', '/bin/true']):
            log.debug('Trying %s', cmd)
            ret = self.run_state(
                'docker_container.run',
                name=name,
                image=self.image,
                command='whoami',
                unless=cmd)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertFalse(ret['changes'])
            self.assertEqual(
                ret['comment'],
                'unless command /bin/true returned exit code of 0'
            )
            self.run_function('docker.rm', [name], force=True)

        for cmd in ('/bin/false', ['/bin/false', 'ls /paththatdoesnotexist']):
            log.debug('Trying %s', cmd)
            ret = self.run_state(
                'docker_container.run',
                name=name,
                image=self.image,
                command='whoami',
                unless=cmd)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertEqual(ret['changes']['Logs'], 'root\n')
            self.assertEqual(
                ret['comment'],
                'Container ran and exited with a return code of 0'
            )
            self.run_function('docker.rm', [name], force=True)

    @container_name
    def test_run_with_creates(self, name):
        '''
        Test docker_container.run with creates. The container should not run
        (and the state should return a True result) if all of the files exist,
        but if if any of the files do not exist the container should run.
        '''
        def _mkstemp():
            fd, ret = tempfile.mkstemp()
            try:
                os.close(fd)
            except OSError as exc:
                if exc.errno != errno.EBADF:
                    raise exc
            else:
                self.addCleanup(os.remove, ret)
                return ret

        bad_file = '/tmp/filethatdoesnotexist'
        good_file1 = _mkstemp()
        good_file2 = _mkstemp()

        for path in (good_file1, [good_file1, good_file2]):
            log.debug('Trying %s', path)
            ret = self.run_state(
                'docker_container.run',
                name=name,
                image=self.image,
                command='whoami',
                creates=path)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertFalse(ret['changes'])
            self.assertEqual(
                ret['comment'],
                'All specified paths in \'creates\' argument exist'
            )
            self.run_function('docker.rm', [name], force=True)

        for path in (bad_file, [good_file1, bad_file]):
            log.debug('Trying %s', path)
            ret = self.run_state(
                'docker_container.run',
                name=name,
                image=self.image,
                command='whoami',
                creates=path)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertEqual(ret['changes']['Logs'], 'root\n')
            self.assertEqual(
                ret['comment'],
                'Container ran and exited with a return code of 0'
            )
            self.run_function('docker.rm', [name], force=True)

    @container_name
    def test_run_replace(self, name):
        '''
        Test the replace and force arguments to make sure they work properly
        '''
        # Run once to create the container
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='whoami')
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret['changes']['Logs'], 'root\n')
        self.assertEqual(
            ret['comment'],
            'Container ran and exited with a return code of 0'
        )

        # Run again with replace=False, this should fail
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='whoami',
            replace=False)
        self.assertSaltFalseReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertFalse(ret['changes'])
        self.assertEqual(
            ret['comment'],
            'Encountered error running container: Container \'{0}\' exists. '
            'Run with replace=True to remove the existing container'.format(name)
        )

        # Run again with replace=True, this should proceed and there should be
        # a "Replaces" key in the changes dict to show that a container was
        # replaced.
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='whoami',
            replace=True)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret['changes']['Logs'], 'root\n')
        self.assertTrue('Replaces' in ret['changes'])
        self.assertEqual(
            ret['comment'],
            'Container ran and exited with a return code of 0'
        )

    @container_name
    def test_run_force(self, name):
        '''
        Test the replace and force arguments to make sure they work properly
        '''
        # Start up a container that will stay running
        ret = self.run_state(
            'docker_container.running',
            name=name,
            image=self.image)
        self.assertSaltTrueReturn(ret)

        # Run again with replace=True, this should fail because the container
        # is still running
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='whoami',
            replace=True,
            force=False)
        self.assertSaltFalseReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertFalse(ret['changes'])
        self.assertEqual(
            ret['comment'],
            'Encountered error running container: Container \'{0}\' exists '
            'and is running. Run with replace=True and force=True to force '
            'removal of the existing container.'.format(name)
        )

        # Run again with replace=True and force=True, this should proceed and
        # there should be a "Replaces" key in the changes dict to show that a
        # container was replaced.
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='whoami',
            replace=True,
            force=True)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret['changes']['Logs'], 'root\n')
        self.assertTrue('Replaces' in ret['changes'])
        self.assertEqual(
            ret['comment'],
            'Container ran and exited with a return code of 0'
        )

    @container_name
    def test_run_failhard(self, name):
        '''
        Test to make sure that we fail a state when the container exits with
        nonzero status if failhard is set to True, and that we don't when it is
        set to False.
        '''
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='/bin/false',
            failhard=True)
        self.assertSaltFalseReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret['changes']['Logs'], '')
        self.assertTrue(
            ret['comment'].startswith(
                'Container ran and exited with a return code of'
            )
        )
        self.run_function('docker.rm', [name], force=True)

        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='/bin/false',
            failhard=False)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret['changes']['Logs'], '')
        self.assertTrue(
            ret['comment'].startswith(
                'Container ran and exited with a return code of'
            )
        )
        self.run_function('docker.rm', [name], force=True)

    @container_name
    def test_run_bg(self, name):
        '''
        Test to make sure that if the container is run in the background, we do
        not include an ExitCode or Logs key in the return. Then check the logs
        for the container to ensure that it ran as expected.
        '''
        ret = self.run_state(
            'docker_container.run',
            name=name,
            image=self.image,
            command='sh -c "sleep 5 && whoami"',
            bg=True)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertTrue('Logs' not in ret['changes'])
        self.assertTrue('ExitCode' not in ret['changes'])
        self.assertEqual(ret['comment'], 'Container was run in the background')

        # Now check the logs. The expectation is that the above asserts
        # completed during the 5-second sleep.
        self.assertEqual(
            self.run_function('docker.logs', [name], follow=True),
            'root\n'
        )



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import requires_salt_modules, skip_if_not_root


@skip_if_not_root
@requires_salt_modules('rabbitmq')
class RabbitModuleTest(ModuleCase):
    '''
    Validates the rabbitmqctl functions.
    To run these tests, you will need to be able to access the rabbitmqctl
    commands.
    '''
    def test_user_exists(self):
        '''
        Find out whether a user exists.
        '''
        ret = self.run_function('rabbitmq.user_exists', ['null_user'])
        self.assertEqual(ret, False)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, Mock, patch

# Import salt libs
import salt.modules.uwsgi as uwsgi


@skipIf(NO_MOCK, NO_MOCK_REASON)
class UwsgiTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/uwsgi'))
        patcher.start()
        self.addCleanup(patcher.stop)
        return {uwsgi: {}}

    def test_uwsgi_stats(self):
        socket = "127.0.0.1:5050"
        mock = MagicMock(return_value='{"a": 1, "b": 2}')
        with patch.dict(uwsgi.__salt__, {'cmd.run': mock}):
            result = uwsgi.stats(socket)
            mock.assert_called_once_with(
                ['uwsgi', '--connect-and-read', '{0}'.format(socket)],
                python_shell=False)
            self.assertEqual(result, {'a': 1, 'b': 2})



# -*- coding: utf-8 -*-
'''
Tests for the salt-run command
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ShellCase
from tests.support.unit import skipIf


class ManageTest(ShellCase):
    '''
    Test the manage runner
    '''
    def test_active(self):
        '''
        jobs.active
        '''
        ret = self.run_run_plus('jobs.active')
        self.assertEqual(ret['return'], {})
        self.assertEqual(ret['out'], [])

    def test_lookup_jid(self):
        '''
        jobs.lookup_jid
        '''
        ret = self.run_run_plus('jobs.lookup_jid', '23974239742394')
        self.assertEqual(ret['return'], {})
        self.assertEqual(ret['out'], [])

    @skipIf(True, 'to be re-enabled when #23623 is merged')
    def test_list_jobs(self):
        '''
        jobs.list_jobs
        '''
        ret = self.run_run_plus('jobs.list_jobs')
        self.assertIsInstance(ret['return'], dict)


class LocalCacheTargetTest(ShellCase):
    '''
    Test that a job stored in the local_cache has target information
    '''

    def test_target_info(self):
        '''
        This is a test case for issue #48734

        PR #43454 fixed an issue where "jobs.lookup_jid" was not working
        correctly with external job caches. However, this fix for external
        job caches broke some inner workings of job storage when using the
        local_cache.

        We need to preserve the previous behavior for the local_cache, but
        keep the new behavior for other external job caches.

        If "savefstr" is called in the local cache, the target data does not
        get written to the local_cache, and the target-type gets listed as a
        "list" type instead of "glob".

        This is a regression test for fixing the local_cache behavior.
        '''
        self.run_salt('minion test.echo target_info_test')
        ret = self.run_run_plus('jobs.list_jobs')
        for item in ret['return'].values():
            if item['Function'] == 'test.echo' and \
                item['Arguments'][0] == 'target_info_test':
                job_ret = item
        tgt = job_ret['Target']
        tgt_type = job_ret['Target-type']

        assert tgt != 'unknown-target'
        assert tgt in ['minion', 'sub_minion']
        assert tgt_type == 'glob'



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import errno

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    mock_open,
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.grub_legacy as grub_legacy
import salt.utils.stringutils
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GrublegacyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.grub_legacy
    '''
    def setup_loader_modules(self):
        return {grub_legacy: {}}

    def test_version(self):
        '''
        Test for Return server version from grub --version
        '''
        mock = MagicMock(return_value='out')
        with patch.dict(grub_legacy.__salt__, {'cmd.run': mock}):
            self.assertEqual(grub_legacy.version(), 'out')

    def test_conf(self):
        '''
        Test for Parse GRUB conf file
        '''
        file_data = IOError(errno.EACCES, 'Permission denied')
        with patch('salt.utils.files.fopen', mock_open(read_data=file_data)), \
                patch.object(grub_legacy, '_detect_conf', return_value='A'):
            self.assertRaises(CommandExecutionError, grub_legacy.conf)

        file_data = salt.utils.stringutils.to_str('\n'.join(['#', 'A B C D,E,F G H']))
        with patch('salt.utils.files.fopen', mock_open(read_data=file_data)), \
                patch.object(grub_legacy, '_detect_conf', return_value='A'):
            conf = grub_legacy.conf()
            assert conf == {'A': 'B C D,E,F G H', 'stanzas': []}, conf



# coding: utf-8

# Python libs
from __future__ import absolute_import
import datetime

# Salt libs
from salt.beacons import telegram_bot_msg

# Salt testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Third-party libs
try:
    import telegram
    HAS_TELEGRAM = True
except ImportError:
    HAS_TELEGRAM = False

import logging
log = logging.getLogger(__name__)


@skipIf(not HAS_TELEGRAM, 'telegram is not available')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class TelegramBotMsgBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.telegram_bot
    '''
    def setup_loader_modules(self):
        return {telegram_bot_msg: {}}

    def test_validate_empty_config(self, *args, **kwargs):
        ret = telegram_bot_msg.validate(None)
        self.assertEqual(ret, (False, ('Configuration for telegram_bot_msg '
                                       'beacon must be a list.')))

    def test_validate_missing_accept_from_config(self, *args, **kwargs):
        ret = telegram_bot_msg.validate([{
            'token': 'bcd'
        }])
        self.assertEqual(ret, (False, ('Not all required configuration for '
                                       'telegram_bot_msg are set.')))

    def test_validate_missing_token_config(self, *args, **kwargs):
        ret = telegram_bot_msg.validate([{
            'accept_from': []
        }])
        self.assertEqual(ret, (False, ('Not all required configuration for '
                                       'telegram_bot_msg are set.')))

    def test_validate_config_not_list_in_accept_from(self, *args, **kwargs):
        ret = telegram_bot_msg.validate([{
            'token': 'bcd',
            'accept_from': {'nodict': "1"}
        }])
        self.assertEqual(ret, (False, ('Configuration for telegram_bot_msg, '
                                       'accept_from must be a list of '
                                       'usernames.')))

    def test_validate_valid_config(self, *args, **kwargs):
        ret = telegram_bot_msg.validate([{
            'token': 'bcd',
            'accept_from': [
                'username'
            ]
        }])
        self.assertEqual(ret, (True, 'Valid beacon configuration.'))

    def test_call_no_updates(self):
        with patch("salt.beacons.telegram_bot_msg.telegram") as telegram_api:
            token = 'abc'
            config = [{
                'token': token,
                'accept_from': ['tester']
            }]
            inst = MagicMock(name='telegram.Bot()')
            telegram_api.Bot = MagicMock(name='telegram', return_value=inst)
            inst.get_updates.return_value = []

            ret = telegram_bot_msg.beacon(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            telegram_api.Bot.assert_called_once_with(token)
            self.assertEqual(ret, [])

    def test_call_telegram_return_no_updates_for_user(self):
        with patch("salt.beacons.telegram_bot_msg.telegram") as telegram_api:
            token = 'abc'
            username = 'tester'
            config = [{
                'token': token,
                'accept_from': [username]
            }]
            inst = MagicMock(name='telegram.Bot()')
            telegram_api.Bot = MagicMock(name='telegram', return_value=inst)

            log.debug('telegram {}'.format(telegram))
            username = 'different_user'
            user = telegram.user.User(id=1, first_name='', username=username)
            chat = telegram.chat.Chat(1, 'private', username=username)
            date = datetime.datetime(2016, 12, 18, 0, 0)
            message = telegram.message.Message(1, user, date=date, chat=chat)
            update = telegram.update.Update(update_id=1, message=message)

            inst.get_updates.return_value = [update]

            ret = telegram_bot_msg.beacon(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            telegram_api.Bot.assert_called_once_with(token)
            self.assertEqual(ret, [])

    def test_call_telegram_returning_updates(self):
        with patch("salt.beacons.telegram_bot_msg.telegram") as telegram_api:
            token = 'abc'
            username = 'tester'
            config = [{
                'token': token,
                'accept_from': [username]
            }]
            inst = MagicMock(name='telegram.Bot()')
            telegram_api.Bot = MagicMock(name='telegram', return_value=inst)

            user = telegram.User(id=1, first_name='', username=username)
            chat = telegram.Chat(1, 'private', username=username)
            date = datetime.datetime(2016, 12, 18, 0, 0)
            message = telegram.Message(1, user, date=date, chat=chat)
            update = telegram.update.Update(update_id=1, message=message)

            inst.get_updates.return_value = [update]

            ret = telegram_bot_msg.beacon(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            telegram_api.Bot.assert_called_once_with(token)
            self.assertTrue(ret)
            self.assertEqual(ret[0]['msgs'][0], message.to_dict())



# -*- coding: utf-8 -*-
'''
Script for copying back xml junit files from tests
'''
from __future__ import absolute_import, print_function
import argparse  # pylint: disable=minimum-python-version
import os
import paramiko
import subprocess

import salt.utils.yaml


class DownloadArtifacts(object):
    def __init__(self, instance, artifacts):
        self.instance = instance
        self.artifacts = artifacts
        self.transport = self.setup_transport()
        self.sftpclient = paramiko.SFTPClient.from_transport(self.transport)

    def setup_transport(self):
        # pylint: disable=minimum-python-version
        config = salt.utils.yaml.safe_load(subprocess.check_output(['bundle', 'exec', 'kitchen', 'diagnose', self.instance]))
        # pylint: enable=minimum-python-version
        state = config['instances'][self.instance]['state_file']
        tport = config['instances'][self.instance]['transport']
        transport = paramiko.Transport((
            state['hostname'],
            state.get('port', tport.get('port', 22))
        ))
        pkey = paramiko.rsakey.RSAKey(
            filename=state.get('ssh_key', tport.get('ssh_key', '~/.ssh/id_rsa'))
        )
        transport.connect(
            username=state.get('username', tport.get('username', 'root')),
            pkey=pkey
        )
        return transport

    def _set_permissions(self):
        '''
        Make sure all xml files are readable by the world so that anyone can grab them
        '''
        for remote, _ in self.artifacts:
            self.transport.open_session().exec_command('sudo chmod -R +r {}'.format(remote))

    def download(self):
        self._set_permissions()
        for remote, local in self.artifacts:
            if remote.endswith('/'):
                for fxml in self.sftpclient.listdir(remote):
                    self._do_download(os.path.join(remote, fxml), os.path.join(local, os.path.basename(fxml)))
            else:
                self._do_download(remote, os.path.join(local, os.path.basename(remote)))

    def _do_download(self, remote, local):
        print('Copying from {0} to {1}'.format(remote, local))
        try:
            self.sftpclient.get(remote, local)
        except IOError:
            print('Failed to copy: {0}'.format(remote))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Jenkins Artifact Download Helper')
    parser.add_argument(
        '--instance',
        required=True,
        action='store',
        help='Instance on Test Kitchen to pull from',
    )
    parser.add_argument(
        '--download-artifacts',
        dest='artifacts',
        nargs=2,
        action='append',
        metavar=('REMOTE_PATH', 'LOCAL_PATH'),
        help='Download remote artifacts',
    )
    args = parser.parse_args()
    downloader = DownloadArtifacts(args.instance, args.artifacts)
    downloader.download()
